@@ -296,7 +296,7 @@ int bbcu_bit_BinatyLut6_Forward
 
 
 // -------------------------------------
-//  Binary LUT
+//  ShuffleModulation
 // -------------------------------------
 
 BBCU_DLL_EXPORT int bbcu_bit_ShuffleModulation_Forward
@@ -313,6 +313,120 @@ BBCU_DLL_EXPORT int bbcu_bit_ShuffleModulation_Forward
         );
 
 
+
+// -------------------------------------
+//  AverageLut
+// -------------------------------------
+
+template <typename T>
+BBCU_DLL_EXPORT int bbcu_AverageLut_Forward
+        (
+            T   const       *dev_x_buf,
+            T               *dev_y_buf,
+            int const       *dev_input_index,
+            int             n,
+            int             node_size,
+            int             frame_size,
+            int             frame_stride,
+            bool            binarize_input,
+            bool            binarize_output,
+            cudaStream_t    streamId=0
+        );
+
+BBCU_DLL_EXPORT int bbcu_bit_AverageLut_Forward
+        (
+            int const       *dev_x_buf,
+            int             *dev_y_buf,
+            int const       *dev_input_index,
+            int             n,
+            int             node_size,
+            int             frame_size,
+            int             frame_stride,
+            cudaStream_t    streamId=0
+        );
+
+template <typename T>
+BBCU_DLL_EXPORT int bbcu_AverageLut_Backward
+        (
+            T   const       *dev_dy_buf,
+            T               *dev_dx_buf,
+            int const       *dev_input_index,
+            int             n,
+            int             input_node_size,
+            int             output_node_size,
+            int             frame_size,
+            int             frame_stride,
+            cudaStream_t    streamId=0
+        );
+
+
+// -------------------------------------
+//  MaxLut
+// -------------------------------------
+
+
+template <typename T>
+BBCU_DLL_EXPORT int bbcu_MaxLut_Forward
+        (
+            T   const       *dev_x_buf,
+            T               *dev_y_buf,
+            int const       *dev_input_index,
+            int             n,
+            int             node_size,
+            int             frame_size,
+            int             frame_stride,
+            bool            binarize_input,
+            bool            binarize_output,
+            cudaStream_t    streamId=0
+        );
+
+BBCU_DLL_EXPORT int bbcu_bit_MaxLut_Forward
+        (
+            int const       *dev_x_buf,
+            int             *dev_y_buf,
+            int const       *dev_input_index,
+            int             n,
+            int             node_size,
+            int             frame_size,
+            int             frame_stride,
+            cudaStream_t    streamId=0
+        );
+
+template <typename T>
+BBCU_DLL_EXPORT int bbcu_MaxLut_Backward
+        (
+            T   const       *dev_x_buf,
+            T   const       *dev_dy_buf,
+            T               *dev_dx_buf,
+            int const       *dev_input_index,
+            int             n,
+            int             input_node_size,
+            int             output_node_size,
+            int             frame_size,
+            int             frame_stride,
+            bool            binarize_input,
+            cudaStream_t    streamId=0
+        );
+
+
+template <typename T>
+BBCU_DLL_EXPORT int bbcu_bit_MaxLut_Backward
+        (
+            int const       *dev_x_buf,
+            T   const       *dev_dy_buf,
+            T               *dev_dx_buf,
+            int const       *dev_input_index,
+            int             n,
+            int             input_node_size,
+            int             output_node_size,
+            int             frame_size,
+            int             x_frame_stride,
+            int             dy_frame_stride,
+            cudaStream_t    streamId=0
+        );
+
+
+
 // -------------------------------------
 //  MicroMlp
 // -------------------------------------
@@ -1394,20 +1508,6 @@ BBCU_DLL_EXPORT int bbcu_MetricsCategoricalAccuracy
         );
 
 
-/*
-BBCU_DLL_EXPORT int bbcu_fp32_AccuracyCategoricalClassification
-        (
-            float const     *dev_y_buf,
-            float const     *dev_t_buf,
-            int             *dev_accuracy,
-            int             node_size,
-            int             frame_size,
-            int             frame_stride,
-            cudaStream_t    streamId = 0
-        );
-*/
-
-
 
 // -------------------------------------
 //  Adam
